package cmdVM

import (
	"errors"

	u "gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/client/utils"
	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/common/params"
	nc "gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/libclient/nexusclient"
	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/libclient/utils"
)

type StartWithCreds struct {
	Name string
}

func (cmd *StartWithCreds) GetName() string {
	return cmd.Name
}

func (cmd *StartWithCreds) GetDesc() []string {
	return []string{
		"Starts one or more VMs with user-defined credentials.",
		"If not the VM's owner: requires VM_START VM access capability or VM_START_ANY user capability."}
}

func (cmd *StartWithCreds) PrintUsage() {
	for _, desc := range cmd.GetDesc() {
		u.PrintlnErr(desc)
	}
	u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
	u.PrintlnErr("USAGE: " + cmd.GetName() + " file.csv")
	u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
	const usage string = `file.csv    3-column CSV file defining the VMs to start and their credentials.
            Content of the 3 columns: VM ID;VM name;password`
	u.PrintlnErr(usage)
}

func (cmd *StartWithCreds) parseCSVFile(csvFile string) ([]string, []string, error) {
	// Column 0: VM IDs
	vmIDs, err := utils.ReadCSVColumn(csvFile, 0)
	if err != nil {
		return nil, nil, err
	}
	// Column 2: passwords
	pwds, err := utils.ReadCSVColumn(csvFile, 2)
	if err != nil {
		return nil, nil, err
	}
	count := len(vmIDs)
	if len(pwds) != count {
		return nil, nil, errors.New("invalid CSV file: all columns must have the same number of entries")
	}
	return vmIDs, pwds, nil
}

func (cmd *StartWithCreds) Run(client *nc.NexusClient, args []string) int {
	argc := len(args)
	if argc != 1 {
		cmd.PrintUsage()
		return 1
	}

	vmIDs, pwds, err := cmd.parseCSVFile(args[0])
	if err != nil {
		u.PrintlnErr(err)
		return 1
	}

	statusCode := 0

	vmArgs := params.VMStartWithCreds{}

	for i, vmID := range vmIDs {
		vmArgs.Pwd = pwds[i]
		err := client.VMStartWithCreds(vmID, vmArgs)
		if err != nil {
			u.PrintlnErr(err)
			statusCode = 1
		} else {
			u.Println("Started VM \"" + vmID + "\"")
		}
	}

	return statusCode
}
