#/bin/bash

run() {
  chroot "$ROOTFS_DIR" $@
}

run_command() {
  local cmd="$@"
  if $VERBOSE; then
      echo $cmd
      bash -c "$cmd"
      EXIT_CODE=$?
  else
      bash -c "$cmd" > /dev/null 2>&1
      EXIT_CODE=$?
  fi
  check_exit_code $EXIT_CODE "Error during command execution: $cmd"
}

run_command_chroot() {
  run_command chroot "$ROOTFS_DIR" $@
}

check_exit_code() {
  EXIT_CODE=$1
  if [ ! $EXIT_CODE -eq 0 ]; then
    echo $2
    exit 1
  fi
}

check_environment_var() {
  if [ -z $SERVER ] || [ -z $CERT ] || [ -z $EXAM_USER ] || [ -z $EXAM_PWD ]; then
    echo "Error: environment variables aren't set" >&2
    echo "SERVER = $SERVER" >&2
    echo "CERT = $CERT" >&2
    echo "EXAM_USER = $EXAM_USER" >&2
    echo "EXAM_PWD = $EXAM_PWD" >&2
    helper
  fi
}

get_total_size() {
    local total=0
    for file in "$@"; do
        if [[ -f "$file" ]]; then
            total=$(( total + $(stat -c%s "$file") ))
        else
            return 1
        fi
    done
    echo "$total"
}

check_inside_container() {
  if [ ! -f "/.dockerenv" ] && [ ! -f "/run/.containerenv" ]; then
    echo "Error: this script must be run inside a Docker or Podman container." >&2
    exit 1
  fi
}

check_dependencies() {
  local dependencies=(debootstrap chroot sed mount umount fallocate mkfs.vfat mksquashfs cryptsetup bash xorriso limine)
  local missing=()

  for cmd in "${dependencies[@]}"; do
    if ! command -v "$cmd" >/dev/null 2>&1; then
      missing+=("$cmd")
    fi
  done

  if [ "${#missing[@]}" -ne 0 ]; then
    echo "Error: the following required tools are missing or not in the PATH:" >&2
    for tool in "${missing[@]}"; do
      echo " - $tool" >&2
    done
    exit 1
  fi
}

find_latest_library() {
  local library_name=$1

  local library_files
  library_files=$(find $ROOTFS_DIR -name "$library_name" 2>/dev/null)

  if [ -z "$library_files" ]; then
    echo "No version available of $library_name." >&2
    return 1
  fi

  local latest_library
  latest_library=$(echo "$library_files" | sort -r | head -n 1)

  realpath "$latest_library"
}
